################################################################
# Library
################################################################
import sys
import os
import socket
import subprocess

import firebase_admin
from firebase_admin import credentials
from firebase_admin import db

################################################################
# Global
################################################################
process = None

################################################################
# Config
################################################################

exe_path = "."
fb_path = exe_path

################################################################
# Main
################################################################

def main():
    # Iinitial log
    print("[Info ] Sensor daemon start")
    # Firebase settings
    key, url = firebase_info(info_pth=fb_path)
    firebase_init(key=key, url=url[0])

    # Setting TCP socket
    sock = init_socket()
    sock.listen(1)

    while True:
        # Wait for a connection
        print ('[Info ] Waiting for a connection')
        connection, client_addr = sock.accept()
        try:
            print ('[Info ] Connection from {}:{}'.format(client_addr[0], client_addr[1]))
            # Receive Data
            operation(client=connection)
        finally:
            # Clean up the connection
            print ('[Info ] Clean the connection')
            connection.close()

################################################################
# Function
################################################################

def get_ipaddr():
    # Set socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    # Get ip address by sending socket
    try:       
        sock.connect(('10.255.255.255', 1))
        ipaddr = sock.getsockname()[0]
    except Exception:
        ipaddr = '127.0.0.1'
    finally:
        sock.close()
    return ipaddr

def init_socket():
    # Setting socket information
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    local_hostname = socket.gethostname()
    local_fqdn = socket.getfqdn()
    ip_address =  get_ipaddr()

    # Print hostname, domain name and IP address
    print("[Info ] Working on %s (%s) with %s" % (local_hostname, local_fqdn, ip_address))

    # Bind the socket to the port 23456
    pi_address = (ip_address, 23456)  
    sock.bind(pi_address)    
    print('[Info ] Starting up on %s port %s' % pi_address)  
    return sock

def receive(client):
    data = None
    while data is None:
        data = client.recv(64)
    return data.decode().split("/")

def operation(client):
    global process
    # Get first command
    cmd = receive(client=client)
    print("[Info ] {}\t\t(cmd)".format(cmd[0]))
    # Start digital sensor
    if "digital" in cmd[0]:
        if "restart" in cmd[0] and len(cmd) == 6:
            if process is not None:
                print("[Info ] Kill {} pid".format(process.pid))
                process.send_signal(1)
            print("[Info ] {}\t\t\t(args)".format(cmd[1]))
            print("[Info ] {}\t\t\t(args)".format(cmd[2]))
            print("[Info ] {}\t\t\t(args)".format(cmd[3]))
            print("[Info ] {}\t\t\t(args)".format(cmd[4]))
            print("[Info ] {}\t\t\t(args)".format(cmd[5]))
            shell_cmd(cmd=cmd)
        elif "start" in cmd[0] and len(cmd) == 6:
            print("[Info ] {}\t\t\t(args)".format(cmd[1]))
            print("[Info ] {}\t\t\t(args)".format(cmd[2]))
            print("[Info ] {}\t\t\t(args)".format(cmd[3]))
            print("[Info ] {}\t\t\t(args)".format(cmd[4]))
            print("[Info ] {}\t\t\t(args)".format(cmd[5]))
            shell_cmd(cmd=cmd)
        elif "stop" in cmd[0]:
            print("[Info ] Kill {} pid".format(process.pid))
            process.send_signal(1)
        else:
            print("[Error] Invalid command!")
    elif "analog" in cmd[0]:
        if "restart" in cmd[0] and len(cmd) == 8:
            if process is not None:
                print("[Info ] Kill {} pid".format(process.pid))
                process.send_signal(1)
            print("[Info ] {}\t\t\t(args)".format(cmd[1]))
            print("[Info ] {}\t\t\t(args)".format(cmd[2]))
            print("[Info ] {}\t\t\t(args)".format(cmd[3]))
            print("[Info ] {}\t\t\t(args)".format(cmd[4]))
            print("[Info ] {}\t\t\t(args)".format(cmd[5]))
            print("[Info ] {}\t\t\t(args)".format(cmd[6]))
            print("[Info ] {}\t\t\t(args)".format(cmd[7]))
            shell_cmd(cmd=cmd)
        elif "start" in cmd[0] and len(cmd) == 8:
            print("[Info ] {}\t\t\t(args)".format(cmd[1]))
            print("[Info ] {}\t\t\t(args)".format(cmd[2]))
            print("[Info ] {}\t\t\t(args)".format(cmd[3]))
            print("[Info ] {}\t\t\t(args)".format(cmd[4]))
            print("[Info ] {}\t\t\t(args)".format(cmd[5]))
            print("[Info ] {}\t\t\t(args)".format(cmd[6]))
            print("[Info ] {}\t\t\t(args)".format(cmd[7]))
            shell_cmd(cmd=cmd)
        elif "stop" in cmd[0]:
            print("[Info ] Kill {} pid".format(process.pid))
            process.send_signal(1)
        else:
            print("[Error] Invalid command!")

def shell_cmd(cmd):
    global process
    # Digital Sensor
    if "digital" in cmd[0]:
        exe = [None] * len(cmd)
        exe[0] = "digital_sensor"
        exe[1] = "--device={}".format(cmd[1])
        exe[2] = "--angle={}".format(cmd[2])
        exe[3] = "--bbox={}".format(cmd[3])
        exe[4] = "--scale={}".format(cmd[4])
        exe[5] = "--pos={}".format(cmd[5])
        process = subprocess.Popen([exe[0], exe[1], exe[2], exe[3], exe[4], exe[5]])
        print("[Info ] PID {}".format(process.pid))
    elif "analog" in cmd[0]:
        exe = [None] * len(cmd)
        exe[0] = "analog_sensor"
        exe[1] = "--device={}".format(cmd[1])
        exe[2] = "--angle={}".format(cmd[2])
        exe[3] = "--unit={}".format(cmd[3])
        exe[4] = "--minvalue={}".format(cmd[4])
        exe[5] = "--maxvalue={}".format(cmd[5])
        exe[6] = "--minangle={}".format(cmd[6])
        exe[7] = "--maxangle={}".format(cmd[7])
        process = subprocess.Popen([exe[0], exe[1], exe[2], exe[3], exe[4], exe[5], exe[6], exe[7]])
        print("[Info ] PID {}".format(process.pid))

# Get Firebase Information
def firebase_info(info_pth):
    # Read JSON file
    key = os.path.join(info_pth, "key.json")
    # Read URL
    with open(os.path.join(info_pth, "url"), "r") as f:
        url = [line[:-1] for line in f]
    return key, url

# Update ip address to firebase
def firebase_init(key, url):
    # Initialize Firebase
    cred = credentials.Certificate(key)
    firebase_admin.initialize_app(cred, {
        'databaseURL' : url
    })
    database = db.reference()
    # Update database
    address =  get_ipaddr()
    database.update({"IP":address})
    print("[Info ] Send ip address to firebase")

if __name__ == "__main__":
    main()
