################################################################
# Library
################################################################
import cv2
import sys
import os
import time
import numpy as np
import argparse
import subprocess

from func import *

################################################################
# Color
################################################################

blue = (255,0,0)
green = (0,255,0)
red = (0,0,255)

################################################################
# Arguments
################################################################

parser = argparse.ArgumentParser(description='Digital Sensor')
parser.add_argument('--device', default='Digital0', type=str,
                    help='Name of this device')

parser.add_argument('--angle', default=90, type=int,
                    help='Choose angle from 0, 90. 180, 270')

parser.add_argument('--bbox', default=1, type=int,
                    help='The number of the bounding box')
                    
parser.add_argument('--scale', default='2.0', type=str,
                    help='Scale of the bounding box')
                    
parser.add_argument('--pos', default='180,160', type=str,
                    help='Coordination of the bounding box')

################################################################
# Config
################################################################

if getattr(sys, 'frozen', False):
    debug = False
    exe_path = os.path.dirname(sys.executable)
    fb_path = exe_path
# For Python File
elif __file__:
    debug = True
    exe_path = "."
    fb_path = os.path.join(exe_path, "firebase")

################################################################
# Main
################################################################

def main(args):   
    # Setting Options
    device = args.device
    angle = args.angle
    bbox = args.bbox
    scale = list(map(float, args.scale.split(',')))
    pos = np.int32(args.pos.split(',')).reshape(-1, 2)
    # Firebase
    key, url = firebase_info(fb_path)
    os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = key
    fb = [firebase(device=device, key=key, url=url[0], idx=idx) for idx in range(bbox)]
    st = Storage(url=url[1])
    # Setting PiCamera
    cap = cv2.VideoCapture(0)
    cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)

    first = True
    while True:
        # Capture Camera Frame
        ret, frame = cap.read()
        frame = cv2_rotate(img=frame, angle=angle)
        if first:
            first = False
            box_image = frame
            for b in range(bbox):
                box_image, image_pth = boxing(img=box_image, scale=scale[b], pos=pos[b])
            box_image, image_pth = coordinate(img=box_image)
            st.upload_image(image=image_pth)
        # Pre-Processing
        preproc = preprocessing(img=frame)
        for b in range(bbox):
            # Crop Bounding Box
            cropframe, debugframe = crop(img=frame, preproc=preproc, scale=scale[b], pos=pos[b])
            # Save Frame PNG file
            preproc_pth, debug_pth = cv2_save(preproc=preproc, bbox=b, cropframe=cropframe, debugframe=debugframe, debug_mode=debug)
            # Run 7-segment OCR
            result = subprocess.run(['ssocr', '-t 35', '-d -1', preproc_pth], stdout=subprocess.PIPE)
            data = result.stdout.decode('utf-8')[:-1]
            # Write Firebase
            fb[b].update(bbox=b, data=data)
        # Wait
        if debug and (cv2.waitKey(1) & 0xFF == ord('q')):
            break

    cap.release()
    cv2.destroyAllWindows()

if __name__ == "__main__":
    # Load arguements
    args = parser.parse_args()
    # Call main function
    main(args)
